@@ -1,3 +1,30 @@
+kw_dflt = function(x, key.pos) {
+	if (is.null(key.pos) || key.pos[1] == 0) # no key:
+		return(lcm(0))
+
+	font_scale = par("ps") / 12
+	if (key.pos[1] == -1)
+		lcm(1.8 * font_scale)
+	else if (key.pos[1] %in% c(2, 4) && (is.character(x[[1]]) || is.factor(x[[1]]))) {
+		strings = if (is.factor(x[[1]]))
+				levels(x[[1]])
+			else
+				x[[1]]
+		lcm(cm(max(strwidth(strings, "inches"))) * 1.3 + font_scale) # cm
+		#lcm(cm(max(strwidth(strings, "inches"))) * 1.3) # cm
+	} else
+		lcm(1.8 * font_scale)
+}
+
+# like cut.default, but only return integers, and allow for duplicate breaks:
+sf_cut = function(values, breaks, include.lowest = TRUE) {
+	i = findInterval(values, breaks, left.open = TRUE)
+	if (include.lowest)
+		i[values == breaks[1]] = 1
+	i[i == 0 | i == length(breaks)] = NA
+	i
+}
+
 #' plot sf object
 #'
 #' plot one or more attributes of an sf object on a map
@@ -10,8 +37,8 @@
 #' @param pal palette function, similar to \link{rainbow}, or palette values; if omitted, \code{sf.colors} is used
 #' @param nbreaks number of colors breaks (ignored for \code{factor} or \code{character} variables)
 #' @param breaks either a numeric vector with the actual breaks, or a name of a method accepted by the \code{style} argument of \link[classInt]{classIntervals}
-#' @param max.plot integer; lower boundary to maximum number of attributes to plot; the default value (9) can be overriden by setting the global option \code{sf_max.plot}, e.g. \code{options(sf_max.plot=2)}
-#' @param key.pos integer; side to plot a color key: 1 bottom, 2 left, 3 top, 4 right; set to \code{NULL} to omit key completely, 0 to only not plot the key, or -1 to select automatically. If multiple columns are plotted in a single function call by default no key is plotted and every submap is stretched individually; if a key is requested (and \code{col} is missing) all maps are colored according to a single key. Auto select depends on plot size, map aspect, and, if set, parameter \code{asp}.
+#' @param max.plot integer; lower boundary to maximum number of attributes to plot; the default value (9) can be overridden by setting the global option \code{sf_max.plot}, e.g. \code{options(sf_max.plot=2)}
+#' @param key.pos numeric; side to plot a color key: 1 bottom, 2 left, 3 top, 4 right; set to \code{NULL} to omit key completely, 0 to only not plot the key, or -1 to select automatically. If multiple columns are plotted in a single function call by default no key is plotted and every submap is stretched individually; if a key is requested (and \code{col} is missing) all maps are colored according to a single key. Auto select depends on plot size, map aspect, and, if set, parameter \code{asp}. If it has lenght 2, the second value, ranging from 0 to 1, determines where the key is placed in the available space (default: 0.5, center).
 #' @param key.width amount of space reserved for the key (incl. labels), thickness/width of the scale bar
 #' @param key.length amount of space reserved for the key along its axis, length of the scale bar
 #' @param pch plotting symbol
@@ -28,6 +55,7 @@
 #' @param extent object with an \code{st_bbox} method to define plot extent; defaults to \code{x}
 #' @param xlim numeric; x-axis limits; overrides \code{extent}
 #' @param ylim numeric; y-axis limits; overrides \code{extent}
+#' @param compact logical; compact sub-plots over plotting space?
 #' @method plot sf
 #' @name plot
 #' @details \code{plot.sf} maximally plots \code{max.plot} maps with colors following from attribute columns,
@@ -71,10 +99,10 @@
 #' 
 #' @export
 plot.sf <- function(x, y, ..., main, pal = NULL, nbreaks = 10, breaks = "pretty",
-		max.plot = if(is.null(n <- getOption("sf_max.plot"))) 9 else n,
-		key.pos = get_key_pos(x, ...), key.length = .618, key.width = lcm(1.8),
+		max.plot = getOption("sf_max.plot", default = 9),
+		key.pos = get_key_pos(x, ...), key.length = .618, key.width = kw_dflt(x, key.pos),
 		reset = TRUE, logz = FALSE, extent = x, xlim = st_bbox(extent)[c(1,3)],
-		ylim = st_bbox(extent)[c(2,4)]) {
+		ylim = st_bbox(extent)[c(2,4)], compact = FALSE) {
 
 	stopifnot(missing(y))
 	nbreaks.missing = missing(nbreaks)
@@ -83,15 +111,17 @@ plot.sf <- function(x, y, ..., main, pal = NULL, nbreaks = 10, breaks = "pretty"
 	dots = list(...)
 	col_missing = is.null(dots$col)
 	breaks_numeric = is.numeric(breaks)
+	reset_layout_needed = reset
 
 	x = swap_axes_if_needed(x)
 
-	opar = par()
+	opar = par(no.readonly = TRUE)
 	if (ncol(x) > 2 && !isTRUE(dots$add)) { # multiple maps to plot...
 		cols = setdiff(names(x), attr(x, "sf_column"))
-		lt = .get_layout(st_bbox(x), min(max.plot, length(cols)), par("din"), key.pos, key.width)
-		key.pos = lt$key.pos
-		layout(lt$m, widths = lt$widths, heights = lt$heights, respect = FALSE)
+		lt = .get_layout(st_bbox(x), min(max.plot, length(cols)), par("din"), key.pos[1], key.width)
+		if (key.pos.missing || key.pos[1] == -1)
+			key.pos = lt$key.pos
+		layout(lt$m, widths = lt$widths, heights = lt$heights, respect = compact)
 
 		if (isTRUE(dots$axes))
 			par(mar = c(2.1, 2.1, 1.2, 0))
@@ -140,7 +170,7 @@ plot.sf <- function(x, y, ..., main, pal = NULL, nbreaks = 10, breaks = "pretty"
 			plot.new()
 
 		# plot key?
-		if (!is.null(key.pos) && key.pos != 0 && col_missing) {
+		if (!is.null(key.pos) && key.pos[1] != 0 && col_missing) {
 			if (is.null(pal))
 				pal = function(n) sf.colors(n, categorical = is.factor(values))
 			colors = if (is.function(pal))
@@ -156,11 +186,10 @@ plot.sf <- function(x, y, ..., main, pal = NULL, nbreaks = 10, breaks = "pretty"
 		}
 
 	} else { # single map, or dots$add == TRUE:
-		if (!isTRUE(dots$add) && reset)
-			layout(matrix(1)) # reset
-		if (ncol(x) == 1) # no attributes to choose colors from: plot geometry
+		if (ncol(x) == 1) { # no attributes to choose colors from: plot geometry
 			plot(st_geometry(x), xlim = xlim, ylim = ylim, ...)
-		else { # generate plot with colors and possibly key
+			reset_layout_needed = FALSE
+		} else { # generate plot with colors and possibly key
 			if (ncol(x) > 2) { # add = TRUE
 				warning("ignoring all but the first attribute")
 				x = x[,1]
@@ -179,7 +208,7 @@ plot.sf <- function(x, y, ..., main, pal = NULL, nbreaks = 10, breaks = "pretty"
 			if (is.null(pal))
 				pal = function(n) sf.colors(n, categorical = is.factor(values))
 			else if (! col_missing)
-				stop("specify only one of col and pal")
+				stop("specify only one of `col' and `pal'")
 
 			if (col_missing) { # compute colors from values:
 				col = if (is.factor(values)) {
@@ -210,8 +239,10 @@ plot.sf <- function(x, y, ..., main, pal = NULL, nbreaks = 10, breaks = "pretty"
 								rep(NA_integer_, length(values))
 							else if (!breaks_numeric && diff(range(values, na.rm = TRUE)) == 0)
 								ifelse(is.na(values), NA_integer_, 1L)
-							else
+							else if (inherits(values, c("POSIXt", "Date")))
 								cut(values, breaks, include.lowest = TRUE)
+							else
+								sf_cut(values, breaks, include.lowest = TRUE)
 						colors = if (is.function(pal))
 								pal(nbreaks)
 							else
@@ -229,7 +260,7 @@ plot.sf <- function(x, y, ..., main, pal = NULL, nbreaks = 10, breaks = "pretty"
 					(is.factor(values) || length(unique(na.omit(values))) > 1 || breaks_numeric) && # 2065
 					length(col) > 1) { # plot key?
 
-				switch(key.pos,
+				switch(key.pos[1],
 					layout(matrix(c(2,1), nrow = 2, ncol = 1),
 						widths = 1, heights = c(1, key.width)), # 1 bottom
 					layout(matrix(c(1,2), nrow = 1, ncol = 2),
@@ -246,7 +277,8 @@ plot.sf <- function(x, y, ..., main, pal = NULL, nbreaks = 10, breaks = "pretty"
 				} else
 					.image_scale(values, colors, breaks = breaks, key.pos = key.pos,
 						key.length = key.length, logz = logz, ...)
-			}
+			} else
+				reset_layout_needed = FALSE # as we didn't call layout()
 			# plot the map:
 			if (!isTRUE(dots$add)) {
 				mar = c(1, 1, 1.2, 1)
@@ -272,11 +304,12 @@ plot.sf <- function(x, y, ..., main, pal = NULL, nbreaks = 10, breaks = "pretty"
 			localTitle(main, ...)
 		}
 	}
-	if (!isTRUE(dots$add) && reset && ncol(x) > 1) { # reset device:
-		layout(matrix(1))
-		desel = which(names(opar) %in% c("cin", "cra", "csi", "cxy", "din", "page", "fig"))
-		par(opar[-desel])
-	}
+	if (!isTRUE(dots$add) && reset) { # reset device: 
+		if (reset_layout_needed) 
+			layout(matrix(1))
+		par(opar)
+	} 
+	invisible()
 }
 
 swap_axes_if_needed = function(x) {
@@ -479,7 +512,7 @@ plot_gc = function(x, pch, cex, bg, border = 1, lty, lwd, col, add) {
 			CURVEPOLYGON = plot.sfc_GEOMETRYCOLLECTION,
 			COMPOUNDCURVE = plot.sfc_GEOMETRYCOLLECTION,
 			GEOMETRYCOLLECTION = plot.sfc_GEOMETRYCOLLECTION,
-			stop(paste("plotting of", class(x)[2], "not yet supported: please file an issue"))
+			stop(paste("plotting of", class(x)[2], "not yet supported: use st_cast?"))
 		)
 		do.call(fn, args)
 	})
@@ -647,8 +680,8 @@ plot_sf = function(x, xlim = NULL, ylim = NULL, asp = NA, axes = FALSE, bgc = pa
 
 
 #' @param n integer; number of colors
-#' @param cutoff.tails numeric, in [0,0.5] start and end values
-#' @param alpha numeric, in [0,1], transparency
+#' @param cutoff.tails numeric, in `[0,0.5]` start and end values
+#' @param alpha numeric, in `[0,1]`, transparency
 #' @param categorical logical; do we want colors for a categorical variable? (see details)
 #' @name plot
 #' @export
@@ -668,7 +701,7 @@ sf.colors = function (n = 10, cutoff.tails = c(0.35, 0.2), alpha = 1, categorica
 			cb = paste0(cb, as.hexmode(ceiling(alpha * 255)))
 		rep(cb, length.out = n)
 	} else {
-		i = seq(0.5 * cutoff.tails[1], 1 - 0.5 * cutoff.tails[2], length = n)
+		i = seq(0.5 * cutoff.tails[1], 1 - 0.5 * cutoff.tails[2], length.out = n)
    		r = ifelse(i < .25, 0, ifelse(i < .57, i / .32 - .78125, 1))
    		g = ifelse(i < .42, 0, ifelse(i < .92, 2 * i - .84, 1))
    		b = ifelse(i < .25, 4 * i, ifelse(i < .42, 1,
@@ -677,40 +710,96 @@ sf.colors = function (n = 10, cutoff.tails = c(0.35, 0.2), alpha = 1, categorica
 	}
 }
 
+# Add text to an existing (base) plot
+#
+#' @param labels character, text to draw (one per row of input)
+#' @name plot
 #' @export
-#' @name stars
-#' @param bb ignore
-#' @param n ignore
-#' @param total_size ignore
-#' @param key.length ignore
-#' @param mfrow length-2 integer vector with number of rows, columns
-.get_layout = function(bb, n, total_size, key.pos, key.length, mfrow = NULL) {
-# return list with "m" matrix, "key.pos", "widths" and "heights" fields
-# if key.pos = -1, it will be a return value, "optimally" placed
+#' @details `text.sf` adds text to an existing base graphic. Text is placed at the centroid of 
+#' each feature in \code{x}. Provide POINT features for further control of placement.
+#' `points.sf` adds point symbols to an existing base graphic. If points of text are not shown
+#' correctly, try setting argument `reset` to `FALSE` in the `plot()` call.
+#' @examples
+#' text(nc, labels = substring(nc$NAME,1,1))
+text.sf = function(x, labels = row.names(x), ...) {
+	text(st_geometry(x), labels = labels, ...)
+}
+
+#' @export
+#' @name plot
+#' @param of_largest_polygon logical, passed on to \link{st_centroid}
+text.sfc = function(x, labels = seq_along(x), ..., of_largest_polygon = FALSE){
+	x = st_centroid(x, of_largest_polygon = of_largest_polygon)
+	xy = st_coordinates(x)
+	text(xy[,1], xy[,2], labels = labels, ...)
+}
+
+#' @name plot
+#' @export
+points.sf = function(x, ...) {
+	points(st_geometry(x), ...)
+}
+
+#' @name plot
+#' @export
+points.sfc = function(x, ..., of_largest_polygon = FALSE) {
+	x = st_centroid(x, of_largest_polygon = of_largest_polygon)
+	xy = st_coordinates(x)
+	points(xy[,1], xy[,2], ...)
+}
+
+# get the aspect ratio of a bounding box, for geodetic coords true scale at mid latitude:
+get_asp = function(bb) {
 	asp = diff(bb[c(2,4)])/diff(bb[c(1,3)])
 	if (!is.finite(asp)) # 0/0
 		asp = 1
 	if (isTRUE(st_is_longlat(bb)))
 		asp = asp / cos(mean(bb[c(2,4)]) * pi /180)
-	if (is.null(mfrow)) {
-		size = function(nrow, n, asp) {
-			ncol = ceiling(n / nrow)
-			xsize = total_size[1] / ncol
-			ysize = xsize  * asp
-			if (xsize * ysize * n > prod(total_size)) {
-				ysize = total_size[2] / nrow
-				xsize = ysize / asp
-			}
-			xsize * ysize
+	asp
+}
+
+#' functions only exported to be used internally by stars
+#' @keywords internal
+#' @name stars
+#' @param bb ignore
+#' @param n ignore
+#' @param total_size ignore
+#' @param key.width ignore
+#' @param key.length ignore
+#' @param mfrow length-2 integer vector with number of rows, columns
+#' @param main main or sub title
+#' @export
+.get_layout = function(bb, n, total_size, key.pos, key.width, mfrow = NULL, main = NULL) {
+# return list with "m" matrix, "key.pos", "widths" and "heights" fields
+# if key.pos = -1 on input, it will be a return value, "optimally" placed
+	asp = get_asp(bb)
+	strip = if (is.character(main))
+			# strheight(main, "inches") 
+			par("cin")[2]
+		else 
+			0.0
+	size = function(nrow, n, asp, strip = 0) { # given nrow n asp, what size does a single tile occupy?
+		ncol = ceiling(n / nrow)
+		xsize = total_size[1] / ncol
+		ysize = xsize  * asp + strip
+		if (xsize * ysize * n > prod(total_size)) {
+			ysize = total_size[2] / nrow - strip
+			xsize = ysize / asp
 		}
-		sz = vapply(1:n, function(x) size(x, n, asp), 0.0)
-		nrow = which.max(sz)
+		c(xsize, ysize)
+	}
+	sz = vapply(1:n, function(nrow) size(nrow, n, asp, strip), c(0.0, 0.0))
+	if (is.null(mfrow)) {
+		nrow = which.max(apply(sz, 2, prod))
 		ncol = ceiling(n / nrow)
 	} else {
 		stopifnot(is.numeric(mfrow), length(mfrow) == 2)
 		nrow = mfrow[1]
 		ncol = mfrow[2]
 	}
+	xsize = sz[1, nrow]
+	ysize = sz[2, nrow]
+	asp = ysize / xsize
 
 	ret = list()
 	ret$mfrow = c(nrow, ncol)
@@ -718,26 +807,26 @@ sf.colors = function (n = 10, cutoff.tails = c(0.35, 0.2), alpha = 1, categorica
 	# the following is right now only used by stars; FIXME:
 	# nocov start
 	ret$key.pos = if (!is.null(key.pos) && key.pos == -1L) { # figure out here: right or bottom?
-			newasp = asp * ncol / nrow # of the composition
-			dispasp = total_size[1] / total_size[2]
-			ifelse(newasp > dispasp, 1, 4) # > or < ? oh dear,
+			newasp = asp * nrow / ncol # of the composition
+			dispasp = total_size[2] / total_size[1]
+			ifelse(newasp > dispasp, 4, 1)
 		} else
 			key.pos
 
 	m = matrix(seq_len(nrow * ncol), nrow, ncol, byrow = TRUE)
-	if (!is.null(ret$key.pos) && ret$key.pos != 0) {
-		k = key.length
+	if (!is.null(ret$key.pos) && ret$key.pos != 0) { # add key row or column:
+		k = key.width
 		n = nrow * ncol + 1
 		switch(ret$key.pos,
-			{ ret$m = rbind(m, n); ret$widths = c(rep(1, ncol)); ret$heights = c(rep(1, nrow), k) },
-			{ ret$m = cbind(n, m); ret$widths = c(k, rep(1, ncol)); ret$heights = c(rep(1, nrow)) },
-			{ ret$m = rbind(n, m); ret$widths = c(rep(1, ncol)); ret$heights = c(k, rep(1, nrow)) },
-			{ ret$m = cbind(m, n); ret$widths = c(rep(1, ncol), k); ret$heights = c(rep(1, nrow)) }
+			{ ret$m = rbind(m, n); ret$widths = c(rep(1, ncol)); ret$heights = c(rep(asp, nrow), k) },
+			{ ret$m = cbind(n, m); ret$widths = c(k, rep(1, ncol)); ret$heights = c(rep(asp, nrow)) },
+			{ ret$m = rbind(n, m); ret$widths = c(rep(1, ncol)); ret$heights = c(k, rep(asp, nrow)) },
+			{ ret$m = cbind(m, n); ret$widths = c(rep(1, ncol), k); ret$heights = c(rep(asp, nrow)) }
 		)
 	} else {
 		ret$m = m
 		ret$widths = rep(1, ncol)
-		ret$heights = rep(1, nrow)
+		ret$heights = rep(asp, nrow)
 	}
 	# nocov end
 	ret
@@ -756,9 +845,7 @@ bb2merc = function(x, cls = "ggmap") { # return bbox in the appropriate "web mer
 	st_bbox(st_transform(pts, merc))
 }
 
-#' functions only exported to be used internally by stars
-#' @name stars
-#' @export
+#' @rdname stars
 #' @param side ignore
 #' @param at ignore
 #' @param labels ignore
@@ -766,6 +853,7 @@ bb2merc = function(x, cls = "ggmap") { # return bbox in the appropriate "web mer
 #' @param lat ignore
 #' @param ndiscr ignore
 #' @param reset ignore
+#' @export
 .degAxis = function (side, at, labels, ..., lon, lat, ndiscr, reset) {
 	if (missing(at))
        	at = axTicks(side)
@@ -779,8 +867,26 @@ bb2merc = function(x, cls = "ggmap") { # return bbox in the appropriate "web mer
 	axis(side, at = at, labels = labels, ...)
 }
 
-#' @name stars
-#' @export
+# find out where to place the legend key:
+# given range r = (a, b), key.length l, key offset o, return a value range that:
+#  (i)  scales such that (b - a) / (y - x) = l, and
+#  (ii) shifts linearly within [x, y] from a = x when o = 0 to b = y when o = 1
+xy_from_r = function(r, l, o) {
+	stopifnot(length(r) == 2, l <= 1, l > 0, o >= 0, o <= 1)
+	r = as.numeric(r)
+	a = r[1]; b = r[2]
+	if (o == 1) {
+		y = b
+		x = b - (b - a)/l
+	} else {
+		i = o / (o - 1)
+		y = (a + (b - a)/l - i * b)/(1 - i)
+		x = i * (y - b) + a
+	}
+	c(x, y)
+}
+
+#' @rdname stars
 #' @param z ignore
 #' @param col ignore
 #' @param breaks ignore
@@ -790,15 +896,23 @@ bb2merc = function(x, cls = "ggmap") { # return bbox in the appropriate "web mer
 #' @param logz ignore
 #' @param ... ignore
 #' @param lab ignore
+#' @param cex.axis see \link{par}
+#' @export
 .image_scale = function(z, col, breaks = NULL, key.pos, add.axis = TRUE,
-		at = NULL, ..., axes = FALSE, key.length, logz = FALSE, lab = "") {
+		at = NULL, ..., axes = FALSE, key.length, logz = FALSE, lab = "", 
+		cex.axis = par("cex.axis")) {
 	if (!is.null(breaks) && length(breaks) != (length(col) + 1))
 		stop("must have one more break than colour")
-	stopifnot(is.character(lab) || is.expression(lab))
+	stopifnot(is.null(lab) || is.character(lab) || is.expression(lab))
 	lab_set = (is.character(lab) && lab != "") || is.expression(lab)
 	zlim = range(z, na.rm = TRUE)
 	if (is.null(breaks))
 		breaks = seq(zlim[1], zlim[2], length.out = length(col) + 1)
+	offset = 0.5
+	if (length(key.pos) == 2) {
+		offset = key.pos[2]
+		key.pos = key.pos[1]
+	}
 	if (is.character(key.length)) {
 		kl = as.numeric(gsub(" cm", "", key.length))
 		sz = if (key.pos %in% c(1,3))
@@ -812,33 +926,32 @@ bb2merc = function(x, cls = "ggmap") { # return bbox in the appropriate "web mer
 		at = pretty(br)
 		at = at[at > br[1] & at < br[2]]
 	}
-	kl_lim = function(r, kl) { m = mean(r); (r - m)/kl + m }
 	if (key.pos %in% c(1,3)) {
 		ylim = c(0, 1)
-		xlim = kl_lim(range(breaks), key.length)
+		xlim = xy_from_r(range(breaks), key.length, offset)
 		mar = c(0, ifelse(axes, 2.1, 1), 0, 1)
 	}
 	if (key.pos %in% c(2,4)) {
-		ylim = kl_lim(range(breaks), key.length)
+		ylim = xy_from_r(range(breaks), key.length, offset)
 		xlim = c(0, 1)
 		mar = c(ifelse(axes, 2.1, 1), 0, 1.2, 0)
 	}
 	mar[key.pos] = 2.1 + 1.5 * lab_set
 	par(mar = mar)
 
-	plot(1, 1, t = "n", ylim = ylim, xlim = xlim, axes = FALSE,
+	plot(1, 1, type = "n", ylim = ylim, xlim = xlim, axes = FALSE,
 		xlab = "", ylab = "", xaxs = "i", yaxs = "i")
-	if (lab != "")
+	if (!is.null(lab) && lab != "")
 		mtext(lab, side = key.pos, line = 2.5, cex = .8)
 	poly = vector(mode="list", length(col))
 	for (i in seq(poly))
 		poly[[i]] = c(breaks[i], breaks[i+1], breaks[i+1], breaks[i])
 	offset = 0.2
 	offs = switch(key.pos,
-		c(0,0,-offset,-offset),
-		c(0,0,-offset,-offset),
-		c(offset,offset,0,0),
-		c(offset,offset,0,0))
+		c(0, 0, -offset, -offset),
+		c(0, 0, -offset, -offset),
+		c(offset, offset, 0, 0),
+		c(offset, offset, 0, 0))
 	for(i in seq_along(poly)) {
 		if (key.pos %in% c(1,3))
 			polygon(poly[[i]], c(0, 0, 1, 1) + offs, col = col[i], border = NA)
@@ -848,9 +961,9 @@ bb2merc = function(x, cls = "ggmap") { # return bbox in the appropriate "web mer
 
 	# box() now would draw around [0,1]:
 	bx = c(breaks[1], rep(tail(breaks, 1), 2), breaks[1])
-	if (key.pos %in% c(1,3))
+	if (key.pos %in% c(1, 3))
 		polygon(bx, c(0, 0, 1, 1) + offs, col = NA, border = 'black')
-	if (key.pos %in% c(2,4))
+	if (key.pos %in% c(2, 4))
 		polygon(c(0, 0, 1, 1) + offs, bx, col = NA, border = 'black')
 
 	labels = if (logz)
@@ -861,51 +974,70 @@ bb2merc = function(x, cls = "ggmap") { # return bbox in the appropriate "web mer
 			TRUE
 
 	if (add.axis)
-		axis(key.pos, at = at, labels = labels)
+		axis(key.pos, at = at, labels = labels, cex.axis = cex.axis)
 }
 
-#' @name stars
-#' @export
+#' @rdname stars
 #' @param key.width ignore
+#' @export
 .image_scale_factor = function(z, col, key.pos, add.axis = TRUE,
-	..., axes = FALSE, key.width, key.length) {
+	..., axes = FALSE, key.width, key.length, cex.axis = par("cex.axis")) {
 
+	to_num = function(x) as.numeric(gsub(" cm", "", x))
 	n = length(z)
-	# TODO:
-	ksz = as.numeric(gsub(" cm", "", key.width)) * 2
+	if ((kw <- to_num(key.width)) < to_num(kw_dflt(list(z), key.pos))) { # cut z until it fits:
+		m = max(nchar(z))
+		while(kw < to_num(kw_dflt(list(z), key.pos))) { # cut z:
+			m = m - 1
+			z = substr(z, 1, m)
+		}
+	}
+	ksz = max(1.5 + max(nchar(z))/2, max(strwidth(z, "inches")) / par("cin")[1]) # in "mar" lines
 	breaks = (0:n) + 0.5
+	offset = 0.5
+	if (length(key.pos) == 2) {
+		offset = key.pos[2]
+		key.pos = key.pos[1]
+	}
 	if (is.character(key.length)) {
-		kl = as.numeric(gsub(" cm", "", key.length))
-		sz = if (key.pos %in% c(1,3))
+		kl = to_num(key.length)
+		sz = if (key.pos %in% c(1, 3))
 				dev.size("cm")[1]
 			else
 				dev.size("cm")[2]
 		key.length = kl/sz
 	}
-	kl_lim = function(r, kl) { m = mean(r); (r - m)/kl + m }
-	if (key.pos %in% c(1,3)) {
+	if (key.pos %in% c(1, 3)) {
 		ylim = c(0, 1)
-		xlim = kl_lim(range(breaks), key.length)
+		xlim = xy_from_r(range(breaks), key.length, offset)
 		mar = c(0, ifelse(axes, 2.1, 1), 0, 1)
 		mar[key.pos] = 2.1
 	} else {
-		ylim = kl_lim(range(breaks), key.length)
+		ylim = xy_from_r(range(breaks), key.length, offset)
 		xlim = c(0, 1)
 		mar = c(ifelse(axes, 2.1, 1), 0, 1.2, 0)
-		#mar[key.pos] = 2.1
-		mar[key.pos] = max(ksz - 1.3, 0.0)
+		mar[key.pos] = ksz
 	}
 	par(mar = mar)
 
 	poly = vector(mode="list", length(col))
 	for (i in seq(poly))
 		poly[[i]] = c(breaks[i], breaks[i+1], breaks[i+1], breaks[i])
-	plot(1, 1, t = "n", ylim = ylim, xlim = xlim, axes = FALSE,
-		xlab = "", ylab = "", xaxs = "i", yaxs = "i")
+
+	tryCatch({
+		plot(1, 1, type = "n", ylim = ylim, xlim = xlim, axes = FALSE,
+			xlab = "", ylab = "", xaxs = "i", yaxs = "i")
+		},
+		error = function(x) {
+			sz = cm(max(strwidth(z, "inches"))) * 1.3 + par("ps")/12 # cm
+			stop(paste0("key.width too small, try key.width = lcm(", signif(sz, 3), ")"), call. = FALSE)
+		}
+	)
+
 	for(i in seq_along(poly)) {
-		if (key.pos %in% c(1,3))
+		if (key.pos %in% c(1, 3))
 			polygon(poly[[i]], c(0, 0, 1, 1), col = col[i], border = NA)
-		if (key.pos %in% c(2,4))
+		if (key.pos %in% c(2, 4))
 			polygon(c(0, 0, 1, 1), poly[[i]], col = col[i], border = NA)
 	}
 
@@ -918,7 +1050,7 @@ bb2merc = function(x, cls = "ggmap") { # return bbox in the appropriate "web mer
 
 	if (add.axis) {
 		opar = par(las = 1)
-		axis(key.pos, at = 1:n, labels = z)
+		axis(key.pos, at = 1:n, labels = z, cex.axis = cex.axis)
 		par(opar)
 	}
 }
