@@ -3,12 +3,8 @@ knitr::opts_chunk$set(fig.height = 4.5)
 knitr::opts_chunk$set(fig.width = 6)
 knitr::opts_chunk$set(collapse = TRUE)
 
-## ----eval=FALSE---------------------------------------------------------------
-#  install.packages("s2")
-
 ## -----------------------------------------------------------------------------
 library(sf)
-sf_use_s2()
 
 ## -----------------------------------------------------------------------------
 library(s2)
@@ -21,26 +17,30 @@ nc = read_sf(system.file("gpkg/nc.gpkg", package="sf"), check_ring_dir = TRUE)
 s2_area(st_as_s2(nc, oriented = TRUE)[1:3]) # no second correction needed here:
 
 ## -----------------------------------------------------------------------------
-g = as_s2_geography(TRUE)
+all(units::drop_units(st_area(nc)) == s2_area(st_as_s2(nc, oriented = FALSE)))
+
+## -----------------------------------------------------------------------------
+g = st_as_sfc("POLYGON FULL", crs = 'EPSG:4326')
 g
 
 ## -----------------------------------------------------------------------------
-co = s2_data_countries()
-oc = s2_difference(g, s2_union_agg(co)) # oceans
-b = s2_buffer_cells(as_s2_geography("POINT(-30 52)"), 9800000) # visible half
-i = s2_intersection(b, oc) # visible ocean
-plot(st_transform(st_as_sfc(i), "+proj=ortho +lat_0=52 +lon_0=-30"), col = 'blue')
+options(s2_oriented = TRUE) # don't change orientation from here on
+co = st_as_sf(s2_data_countries())
+oc = st_difference(g, st_union(co)) # oceans
+b = st_buffer(st_as_sfc("POINT(-30 52)", crs = 'EPSG:4326'), 9800000) # visible half
+i = st_intersection(b, oc) # visible ocean
+plot(st_transform(i, "+proj=ortho +lat_0=52 +lon_0=-30"), col = 'blue')
 
 ## -----------------------------------------------------------------------------
-s2_area(oc) / s2_area(g)
+st_area(oc) / st_area(g)
 
 ## -----------------------------------------------------------------------------
-a = as_s2_geography("POINT(0 0)")
-b = as_s2_geography("POLYGON((0 0,1 0,1 1,0 1,0 0))")
-s2_intersects(a, b, s2_options(model = "open")) 
-s2_intersects(a, b, s2_options(model = "closed"))
-s2_intersects(a, b, s2_options(model = "semi-open")) # a toss
-s2_intersects(a, b) # default: semi-open
+a = st_as_sfc("POINT(0 0)", crs = 'EPSG:4326')
+b = st_as_sfc("POLYGON((0 0,1 0,1 1,0 1,0 0))", crs = 'EPSG:4326')
+st_intersects(a, b, model = "open")
+st_intersects(a, b, model = "closed")
+st_intersects(a, b, model = "semi-open") # a toss
+st_intersects(a, b) # default: closed
 
 ## -----------------------------------------------------------------------------
 fiji = s2_data_countries("Fiji")
@@ -58,6 +58,7 @@ sf_use_s2(FALSE)
 sf_use_s2(TRUE)
 
 ## ----eval=require("lwgeom", quietly = TRUE)-----------------------------------
+options(s2_oriented = FALSE) # correct orientation from here on
 library(sf)
 library(units)
 nc = read_sf(system.file("gpkg/nc.gpkg", package="sf"))
@@ -77,7 +78,7 @@ sf_use_s2(FALSE)
 l2 = st_length(nc_ls)
 plot(l1 , l2)
 abline(0, 1)
-summary((l1-l2)/l1)
+summary((l1 - l2)/l1)
 
 ## -----------------------------------------------------------------------------
 sf_use_s2(TRUE)
@@ -86,7 +87,7 @@ sf_use_s2(FALSE)
 d2 = st_distance(nc, nc[1:10,])
 plot(as.vector(d1), as.vector(d2))
 abline(0, 1)
-summary(as.vector(d1)-as.vector(d2))
+summary(as.vector(d1) - as.vector(d2))
 
 ## -----------------------------------------------------------------------------
 sf_use_s2(TRUE)
@@ -98,14 +99,14 @@ st_intersects(nc[1:3,], nc[1:3,], model = "semi-open") # only self-intersections
 uk = s2_data_countries("United Kingdom")
 class(uk)
 uk_sfc = st_as_sfc(uk) 
-uk_buffer = s2_buffer_cells(uk, distance = 20000)
-uk_buffer2 = s2_buffer_cells(uk, distance = 20000, max_cells = 10000)
-uk_buffer3 = s2_buffer_cells(uk, distance = 20000, max_cells = 100)
+uk_buffer = st_buffer(uk_sfc, dist = 20000)
+uk_buffer2 = st_buffer(uk_sfc, dist = 20000, max_cells = 10000)
+uk_buffer3 = st_buffer(uk_sfc, dist = 20000, max_cells = 100)
 class(uk_buffer)
 plot(uk_sfc)
-plot(st_as_sfc(uk_buffer))
-plot(st_as_sfc(uk_buffer2))
-plot(st_as_sfc(uk_buffer3))
+plot(uk_buffer)
+plot(uk_buffer2)
+plot(uk_buffer3)
 uk_sf = st_as_sf(uk) 
 
 ## -----------------------------------------------------------------------------
