@@ -4,16 +4,15 @@ suppressPackageStartupMessages(library(units))
 if (utils::packageVersion("units") >= "0.5-0")
 	units_options(auto_convert_names_to_symbols = FALSE)
 
-ll = st_crs(4326)
-sf:::crs_parameters(ll)$ud_unit
+st_crs(4326)$ud_unit
 
-u = names(sf:::udunits_from_proj)
+u = names(sf:::udunits_from_proj)[1:21]
 
 unrecognized = NULL
 out = sapply(u, function(x) { 
-  p4s = paste0("+proj=aea +units=", x)
+  p4s = paste0("+proj=laea +units=", x)
   cat(x, ": ")
-  ret = try(sf:::crs_parameters(st_crs(p4s))$ud_unit, silent = TRUE)
+  ret = try(st_crs(p4s)$ud_unit, silent = TRUE)
   if (! inherits(ret, "try-error"))
   	print(ret)
   else
